<?php

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) exit;

// Check if class already exists

//if (!class_exists("NNRobots_Optin_Fire_DB")):

/**
 * This class will perform all databse related tasks to manage all optin fires
 *
 * @since 1.0.0
 *
 */
include_once(NNROBOTS_OPTIN_FIRE_PLUGIN_DIR . "/admin/model/list-table.php");

/**
 * NNRobots_Optin_Fire_DB class.
 */
class NNRobots_Optin_Fire_DB {

	/**
	 * table_name
	 *
	 * (default value: 'nnrobots_optin_fire')
	 *
	 * @var string
	 * @access private
	 * @static
	 */
	private static $table_name = 'nnrobots_optin_fire';

	/**
	 * Contructor of this class.
	 *
	 * @since 1.0.0
	 *
	 * @param 	N/A
	 * @return 	Instance
	 */
	function __construct() {

		self::createTable();

	}

	/**
	 * Creates the table used to store all optin fire data
	 *
	 * @access public
	 * @return void
	 */
	static function createTable() {
		global $wpdb;

		$wpdb->query("
			CREATE TABLE IF NOT EXISTS `" . self::getTableName() . "` (
				`id` int(11) NOT NULL AUTO_INCREMENT,
				`type` int(11),
				`name` varchar(60) NOT NULL DEFAULT '',
				`title` mediumtext NOT NULL,
				`text` mediumtext NOT NULL,
				`image` int(11) NOT NULL,
				`news_letter` longtext NOT NULL DEFAULT '',
				`template` longtext NOT NULL,
				`trigger` longtext NOT NULL,
				`style` longtext NOT NULL,
				`active` varchar(8) NOT NULL DEFAULT 'Active',
				`args` longtext,
				PRIMARY KEY (`id`)
			) ENGINE=InnoDB DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci AUTO_INCREMENT=1;
		");
	}

	/**
	 * Add data
	 *
	 * @since 1.0.0
	 *
	 * @param	data to be added
	 * @return	false if error, otherwise nothing
	 */
	public function add_data($val) {

		global $wpdb;

		if(!is_array($val) || $val['name'] == "")
			return false;

		$result = $wpdb->query($wpdb->prepare("INSERT INTO `" . self::getTableName() . "` (
				`type`,
				`name`,
				`title`,
				`text`,
				`image`,
				`news_letter`,
				`template`,
				`trigger`,
				`style`,
				`active`,
				`args`
			) VALUES (%s, %s, %s, %s, %d, %s, %s, %s, %s, %s, %s)",
				$val['type'],
				$val['name'],
				$val['title'],
				$val['text'],
				$val['image'],
				serialize($val['news_letter']),
				serialize($val['template']),
				serialize($val['trigger']),
				serialize($val['style']),
				$val['active'],
				serialize($val['args'])
			)
		);

		return $wpdb->insert_id;
	}

	/**
	 * Update data
	 *
	 * @since 1.0.0
	 *
	 * @param	data to be updated
	 * @return	false if error, otherwise nothing
	 */
	public function update_data($val) {

		global $wpdb;

		if(!is_array($val) || $val['id'] == "")
			return false;

		$result = $wpdb->query($wpdb->prepare(
			"UPDATE `" . self::getTableName() . "` SET
				`type` = %s,
				`name` = %s,
				`title` = %s,
				`text` = %s,
				`image` = %d,
				`news_letter` = %s,
				`template` = %s,
				`trigger` = %s,
				`style` = %s,
				`active` = %s,
				`args` = %s
			WHERE id = %d",
				$val['type'],
				$val['name'],
				$val['title'],
				$val['text'],
				$val['image'],
				serialize($val['news_letter']),
				serialize($val['template']),
				serialize($val['trigger']),
				serialize($val['style']),
				$val['active'],
				serialize($val['args']),
				$val['id']
			)
		);

		return $result;
	}

	/**
	 * Get all optin fires
	 *
	 * @since 1.0.0
	 *
	 * @param	id
	 * @return	data of optin_fire
	 */
	public function get_optin_fires() {

		global $wpdb;

		$data = $wpdb->get_results("SELECT * FROM `" . self::getTableName() . "`", 'ARRAY_A');

		$vals = array();

		foreach($data as $row) {
			$val = array();

			$val["id"]           = $row["id"];
			$val["type"]         = $row["type"];
			$val["name"]         = $row["name"];
			$val["title"]        = $row["title"];
			$val["text"]         = $row["text"];
			$val["image"]        = $row["image"];
			$val["news_letter"]  = $row["news_letter"];
			$val["template"]     = $row["template"];
			$val["trigger"]      = $row["trigger"];
			$val["style"]        = $row["style"];
			$val["active"]       = $row["active"];
			$val["args"]         = $row["args"];

			$val['news_letter']    = self::mb_unserialize($val['news_letter']);
			$val['template']       = self::mb_unserialize($val['template']);
			$val['trigger']        = self::mb_unserialize($val['trigger']);
			$val['style']          = self::mb_unserialize($val['style']);
			$val['args']           = self::mb_unserialize($val['args']);

			$vals[] = $val;
		}

		return $vals;
	}

	/**
	 * Get the data from id
	 *
	 * @since 1.0.0
	 *
	 * @param	id
	 * @return	data of optin_fire
	 */
	public function get_data_from_id($id) {

		global $wpdb;

		$data = $wpdb->get_results($wpdb->prepare("SELECT * FROM `" . self::getTableName() . "` WHERE `id` = %d", (int) $id), 'ARRAY_A');

		foreach($data as $row) {
			$val = array();

			$val["id"]           = $row["id"];
			$val["type"]         = $row["type"];
			$val["name"]         = $row["name"];
			$val["title"]        = $row["title"];
			$val["text"]         = $row["text"];
			$val["image"]        = $row["image"];
			$val["news_letter"]  = $row["news_letter"];
			$val["template"]     = $row["template"];
			$val["trigger"]      = $row["trigger"];
			$val["style"]        = $row["style"];
			$val["active"]       = $row["active"];
			$val["args"]         = $row["args"];

			$val['news_letter']    = self::mb_unserialize($val['news_letter']);
			$val['template']       = self::mb_unserialize($val['template']);
			$val['trigger']        = self::mb_unserialize($val['trigger']);
			$val['style']          = self::mb_unserialize($val['style']);
			$val['args']           = self::mb_unserialize($val['args']);

		}

		if (isset($val)) {
			return $val;
		}

		return null;
	}

	/**
	 * Get optins based on type
	 *
	 * @access public
	 * @static
	 * @param array $types (default: array(1)
	 * @param mixed 2
	 * @param mixed 3
	 * @param mixed 4
	 * @param mixed 5)
	 * @return void
	 */
	static function get_by_type($types = array(1,2,3,4,5)) {

		global $wpdb;

		$query = "SELECT * FROM `" . self::getTableName() . "` WHERE `active` = 'Active' AND (";

		foreach ( $types as $type ) {
			$query .= "`type` = '" . $type . "' OR ";
		}

		$query = self::str_lreplace(' OR ', '', $query);

		$query .= ')';

		$optins = $wpdb->get_results($query);

		$valid_optins = array();

		if ( $optins ) {

			foreach ( $optins as $optin ) {

				$optin->news_letter = self::mb_unserialize($optin->news_letter);
				$optin->template    = self::mb_unserialize($optin->template);
				$optin->trigger     = self::mb_unserialize($optin->trigger);
				$optin->style       = self::mb_unserialize($optin->style);
				$optin->args        = self::mb_unserialize($optin->args);

				$valid_optins[] = (array) $optin;

			}

			return $valid_optins;
		}

		return array();
	}

	/**
	 * Get all the active optins
	 *
	 * @access public
	 * @return void
	 */
	public function get_active() {

		global $wpdb;

		$data = $wpdb->get_results("SELECT * FROM `" . self::getTableName() . "` WHERE `active` = 'Active'", 'ARRAY_A');

		$vals = array();

		foreach($data as $row) {
			$val = array();

			$val["id"]           = $row["id"];
			$val["type"]         = $row["type"];
			$val["name"]         = $row["name"];
			$val["title"]        = $row["title"];
			$val["text"]         = $row["text"];
			$val["image"]        = $row["image"];
			$val["news_letter"]  = $row["news_letter"];
			$val["template"]     = $row["template"];
			$val["trigger"]      = $row["trigger"];
			$val["style"]        = $row["style"];
			$val["active"]       = $row["active"];
			$val["args"]         = $row["args"];

			$val['news_letter']    = self::mb_unserialize($val['news_letter']);
			$val['template']       = self::mb_unserialize($val['template']);
			$val['trigger']        = self::mb_unserialize($val['trigger']);
			$val['style']          = self::mb_unserialize($val['style']);
			$val['args']           = self::mb_unserialize($val['args']);

			$vals[] = $val;

		}

		return $vals;

	}

	/**
	 * Get all the inactive optins
	 *
	 * @access public
	 * @return void
	 */
	public function get_inactive() {

		global $wpdb;

		$data = $wpdb->get_results("SELECT * FROM `" . self::getTableName() . "` WHERE `active` = 'Inactive'", 'ARRAY_A');

		$vals = array();

		foreach($data as $row) {
			$val = array();

			$val["id"]           = $row["id"];
			$val["type"]         = $row["type"];
			$val["name"]         = $row["name"];
			$val["title"]        = $row["title"];
			$val["text"]         = $row["text"];
			$val["image"]        = $row["image"];
			$val["news_letter"]  = $row["news_letter"];
			$val["template"]     = $row["template"];
			$val["trigger"]      = $row["trigger"];
			$val["style"]        = $row["style"];
			$val["active"]       = $row["active"];
			$val["args"]         = $row["args"];

			$val['news_letter']    = self::mb_unserialize($val['news_letter']);
			$val['template']       = self::mb_unserialize($val['template']);
			$val['trigger']        = self::mb_unserialize($val['trigger']);
			$val['style']          = self::mb_unserialize($val['style']);
			$val['args']           = self::mb_unserialize($val['args']);

			$vals[] = $val;

		}

		return $vals;

	}

	/**
	 * Set to active or inactive
	 *
	 * @since 1.0.0
	 *
	 * @param	record id
	 * @param	value to set
	 * @return	false if error, otherwise nothing
	 */
	public function set_active($id, $active) {

		global $wpdb;

		$result = $wpdb->query($wpdb->prepare("UPDATE `" . self::getTableName() . "` SET active = %s WHERE id = %d", $active, $id));
	}

	/**
	 * Delete data from id
	 *
	 * @since 1.0.0
	 *
	 * @param	id
	 * @return	N/A
	 */
	public function delete_data($id) {

		global $wpdb;

		$result = $wpdb->query($wpdb->prepare("DELETE FROM `" . self::getTableName() . "` WHERE id = %d", $id));
	}

	/**
	 * Returns the table name with multisite support
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	public static function getTableName() {

		global $wpdb;

		return $wpdb->prefix . self::$table_name;
	}

	/**
	 * Mulit-byte Unserialize
	 *
	 * UTF-8 will screw up a serialized string
	 *
	 * @access private
	 * @param string
	 * @return string
	 */
	static function mb_unserialize($string) {

		if ( !is_string($string) ) {
			return $string;
		}

	    $string2 = preg_replace_callback(
	        '!s:(\d+):"(.*?)";!s',
	        function($m){
	            $len = strlen($m[2]);
	            $result = "s:$len:\"{$m[2]}\";";
	            return $result;

	        },
	        $string);
	    return unserialize($string2);
	}

	/**
	 * Finds the last occurcence of a string and replaces it with another string
	 *
	 * @access public
	 * @param mixed $search
	 * @param mixed $replace
	 * @param mixed $subject
	 * @return void
	 */
	static function str_lreplace($search, $replace, $subject) {
	    $pos = strrpos($subject, $search);

	    if($pos !== false)
	    {
	        $subject = substr_replace($subject, $replace, $pos, strlen($search));
	    }

	    return $subject;
	}
}

//endif;
?>