<?php

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) exit;

// Check if class already exists

//if (!class_exists("NNRobots_Email_Signups_OF")):

// AJAX functions for email signups

add_action( 'wp_ajax_nnr_of_add_email', 		array('NNRobots_Email_Signups_OF', 'addEmail'));
add_action( 'wp_ajax_nopriv_nnr_of_add_email', 	array('NNRobots_Email_Signups_OF', 'addEmail'));

/**
 * NNRobots_Email_Signups_OF class.
 *
 * This class is used to record all stats for optin fires.
 */
class NNRobots_Email_Signups_OF {

	/**
	 * table_name
	 *
	 * (default value: 'nnrobots_optin_fire_stats')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	public static $table_name = 'nnrobots_optin_fire_emails';

	/**
	 * Create Table when this class is loaded
	 *
	 * @access public
	 * @return void
	 */
	function __construct() {
		self::createTable();
	}

	/**
	 * Creates the Stats table if it does not exist
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function createTable(){

		global $wpdb;

		$wpdb->query("
			CREATE TABLE IF NOT EXISTS " . self::getTableName() . " (
				`id` int(11) NOT NULL AUTO_INCREMENT,
				`date` datetime NOT NULL,
				`optin_fire_id` int(11) NOT NULL,
				`email` varchar(255) NOT NULL UNIQUE,
				`first_name` varchar(50) NOT NULL DEFAULT '',
				`last_name` varchar(50) NOT NULL DEFAULT '',
				PRIMARY KEY (`id`)
			) ENGINE=InnoDB DEFAULT CHARSET=latin1 AUTO_INCREMENT=1;
		");

	}

	/**
	 * Get the emails over a date range in all time
	 *
	 * @access public
	 * @static
	 * @param mixed $start
	 * @param mixed $end
	 * @param mixed $id (default: null)
	 * @return void
	 */
	static function getEmails($start, $end, $email = null, $select = '*') {

		$query = null;

		global $wpdb;

		// All Optins, All Time

		if ($start == null && $end == null && $email == null) {
			$query = 'SELECT ' . $select . ' FROM ' . self::getTableName();
		}

		// All Optins, After Date

		else if ($start != null && $end == null && $email == null) {
			$query = $wpdb->prepare('SELECT ' . $select . ' FROM ' . self::getTableName() . ' WHERE `date` = %s', $start);
		}

		// All Optins, Date Range

		else if ($start != null && $end != null && $email == null) {
			$query = $wpdb->prepare('SELECT ' . $select . ' FROM ' . self::getTableName() . ' WHERE `date` >= %s AND `date` <= %s', $start, $end);
		}

		// Single Email

		else if ($start == null && $end == null && $email != null) {
			$query = $wpdb->prepare('SELECT ' . $select . ' FROM ' . self::getTableName() . ' WHERE `email` = %d', $email);
		}

		// No query was created

		if (!isset($query)) {
			return false;
		}

		$result = $wpdb->get_results($query, 'ARRAY_A');

		return $result;
	}

	/**
	 * Record an impression made by a optin fire
	 * This function is called via PHP.
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function addEmail() {

		// No First Name

		if (!isset($_POST['first_name'])) {
			$_POST['first_name'] = '';
		}

		// No Last Name

		if (!isset($_POST['last_name'])) {
			$_POST['last_name'] = '';
		}

		// Could not find Optin ID

		if (!isset($_POST['optin_fire_id']) || $_POST['optin_fire_id'] == '') {
			echo json_encode(array(
				'id'		=> $_POST['optin_fire_id'],
				'status'	=> 'warning',
				'message'	=> __('Could not find Optin ID.', NNRobots_Optin_Fire::$text_domain)
			));
			die();
		}

		include_once(NNROBOTS_OPTIN_FIRE_PLUGIN_DIR . '/admin/model/db.php');

		$nnrobots_optin_fire_db = new NNRobots_Optin_Fire_DB();
		$optin_fire = $nnrobots_optin_fire_db->get_data_from_id($_POST['optin_fire_id']);

		$success_action = stripcslashes($optin_fire['news_letter']['success_action']);
		$success_mesage = stripcslashes($optin_fire['news_letter']['success_message']);
		$success_url 	= stripcslashes($optin_fire['news_letter']['success_url']);

		// No Email

		if (!isset($_POST['email']) || $_POST['email'] == '') {
			echo json_encode(array(
				'id'		=> $_POST['optin_fire_id'],
				'status'	=> 'warning',
				'message'	=> __('No Email address provided.', NNRobots_Optin_Fire::$text_domain)
			));
			die();
		}

		// Invalid Email Address

		if (!filter_var($_POST['email'], FILTER_VALIDATE_EMAIL)) {
			echo json_encode(array(
				'id'		=> $_POST['optin_fire_id'],
				'status'	=> 'warning',
				'message'	=> __('Invalid Email provided.', NNRobots_Optin_Fire::$text_domain)
			));
			die();
		}

		// Add email to WordPress table

		if ($_POST['type'] == 'wordpress') {

			global $wpdb;
			$today = date(NNROBOTS_OPTIN_FIRE_ENTRY_DATE_FORMAT);

			$query = $wpdb->prepare('INSERT IGNORE INTO ' . self::getTableName() . ' (
				`date`,
				`optin_fire_id`,
				`email`,
				`first_name`,
				`last_name`
				) VALUES (%s, %d, %s, %s, %s)',
				$today,
				$_POST['optin_fire_id'],
				$_POST['email'],
				$_POST['first_name'],
				$_POST['last_name']
			);

			$result = $wpdb->query($query);

			if ($result) {

				// Add Cookie to let us know not to show this optin fire ever again.

				self::setEmailCookie($optin_fire['args'], NNRobots_Optin_Fire::$prefix . 'email_success_' . $_POST['optin_fire_id']);

				echo json_encode(array(
					'id'				=> $_POST['optin_fire_id'],
					'status'			=> 'check',
					'success_action'	=> $success_action,
					'url'				=> $success_url,
					'message'			=> $success_mesage,
					'conversion'		=> self::recordConversion($_POST['optin_fire_id']),
				));

				die();

			} else {
				echo json_encode(array(
					'id'		=> $_POST['optin_fire_id'],
					'status'	=> 'check',
					'message'	=> 'We already have your email!'
				));

				die();
			}
		}

		// MailChimp

		else if ($_POST['type'] == 'mailchimp') {

			require_once(NNROBOTS_OPTIN_FIRE_PLUGIN_DIR . '/core/includes/mailchimp/MailChimp.php');

			$nnrobots_optin_fire_db = new NNRobots_Optin_Fire_DB();
			$optin_fire = $nnrobots_optin_fire_db->get_data_from_id($_POST['optin_fire_id']);

			if (!isset($optin_fire['news_letter']['mailchimp']['api_key']) || $optin_fire['news_letter']['mailchimp']['api_key'] == '') {
				echo json_encode(array(
					'id'		=> $_POST['optin_fire_id'],
					'status'	=> 'warning',
					'message'	=> __('Mailchimp account is not setup properly.'),
				));

				die();
			}

			if (!isset($optin_fire['news_letter']['mailchimp']['list']) || $optin_fire['news_letter']['mailchimp']['list'] == '') {
				echo json_encode(array(
					'id'		=> $_POST['optin_fire_id'],
					'status'	=> 'warning',
					'message'	=> __('MailChimp: No list specified.'),
				));

				die();
			}

			if ( !isset($optin_fire['news_letter']['mailchimp']['optin']) ) {
				$optin_fire['news_letter']['mailchimp']['optin'] = true;
			}

			$MailChimp = new WPS_MailChimp($optin_fire['news_letter']['mailchimp']['api_key']);
			$result = $MailChimp->call('lists/subscribe', array(
                'id'                => $optin_fire['news_letter']['mailchimp']['list'],
                'email'             => array('email'=>$_POST['email']),
                'merge_vars'        => array('FNAME'=>$_POST['first_name'], 'LNAME'=>$_POST['last_name']),
                'double_optin'      => $optin_fire['news_letter']['mailchimp']['optin'],
                'update_existing'   => false,
                'replace_interests' => false,
                'send_welcome'      => true,
            ));

            if ($result) {

	            if (isset($result['email'])) {

		            // Add Cookie to let us know not to show this optin fire ever again.

					self::setEmailCookie($optin_fire['args'], NNRobots_Optin_Fire::$prefix . 'email_success_' . $_POST['optin_fire_id']);

					echo json_encode(array(
						'id'				=> $_POST['optin_fire_id'],
						'status'			=> 'check',
						'success_action'	=> $success_action,
						'url'				=> $success_url,
						'message'			=> $success_mesage,
						'conversion'		=> self::recordConversion($_POST['optin_fire_id']),
					));

					die();
	            }

	            else if (isset($result['status']) && $result['status'] == 'error') {
					echo json_encode(array(
						'id'		=> $_POST['optin_fire_id'],
						'status'		=> 'warning',
						'message'		=> $result['error'],
					));

					die();
	            }
            } else {

	            echo json_encode(array(
		            'id'		=> $_POST['optin_fire_id'],
					'status'	=> 'warning',
					'message'	=> __('Unable to subscribe.'),
				));

				die();
            }
		}

		// Add email to Aweber

		else if ($_POST['type'] == 'aweber') {

			require_once(NNROBOTS_OPTIN_FIRE_PLUGIN_DIR . '/core/includes/aweber/aweber_api.php');

			$nnrobots_optin_fire_db = new NNRobots_Optin_Fire_DB();
			$optin_fire = $nnrobots_optin_fire_db->get_data_from_id($_POST['optin_fire_id']);

			$aweber = new AWeberAPI($optin_fire['news_letter']['aweber']['consumer_key'], $optin_fire['news_letter']['aweber']['consumer_secret']);

			try {
				$account = $aweber->getAccount($optin_fire['news_letter']['aweber']['access_key'], $optin_fire['news_letter']['aweber']['access_secret']);
				$list = $account->loadFromUrl('/accounts/' . $account->id . '/lists/' . $optin_fire['news_letter']['aweber']['list_id']);

				$subscriber = array(
					'email' 	=> $_POST['email'],
					'name'		=> $_POST['first_name'] . ' ' . $_POST['last_name'],
					'ip' 		=> $_SERVER['REMOTE_ADDR']
				);

				$newSubscriber = $list->subscribers->create($subscriber);

				// Add Cookie to let us know not to show this optin fire ever again.

				self::setEmailCookie($optin_fire['args'], NNRobots_Optin_Fire::$prefix . 'email_success_' . $_POST['optin_fire_id']);

				echo json_encode(array(
					'id'				=> $_POST['optin_fire_id'],
					'status'			=> 'check',
					'success_action'	=> $success_action,
					'url'				=> $success_url,
					'message'			=> $success_mesage,
					'conversion'		=> self::recordConversion($_POST['optin_fire_id']),
				));

				die();

			} catch (AWeberAPIException $exc) {
				echo json_encode(array(
					'id'		=> $_POST['optin_fire_id'],
					'status'	=> 'warning',
					'message'	=> $exc->message,
				));

				die();
			}
		}

		// Add email to Get Response

		else if ($_POST['type'] == 'getresponse') {

			require_once(NNROBOTS_OPTIN_FIRE_PLUGIN_DIR . '/core/includes/getresponse/jsonRPCClient.php');

			$nnrobots_optin_fire_db = new NNRobots_Optin_Fire_DB();
			$optin_fire = $nnrobots_optin_fire_db->get_data_from_id($_POST['optin_fire_id']);

			$api = new jsonRPCClient('http://api2.getresponse.com');

			try {
				$api->add_contact(
					$optin_fire['news_letter']['getresponse']['api_key'],
				    array (
				        'campaign'  => $optin_fire['news_letter']['getresponse']['campaign'],
				        'name'      => $_POST['first_name'] . ' ' . $_POST['last_name'],
				        'email'     => $_POST['email'],
				    )
				);

				// Add Cookie to let us know not to show this optin fire ever again.

				self::setEmailCookie($optin_fire['args'], NNRobots_Optin_Fire::$prefix . 'email_success_' . $_POST['optin_fire_id']);

				echo json_encode(array(
					'id'				=> $_POST['optin_fire_id'],
					'status'			=> 'check',
					'success_action'	=> $success_action,
					'url'				=> $success_url,
					'message'			=> $success_mesage,
					'conversion'		=> self::recordConversion($_POST['optin_fire_id']),
				));

				die();

			} catch (RuntimeException $exc) {

				$msg = $exc->getMessage();
				$msg = substr($msg, 0, strpos($msg, ";"));

				echo json_encode(array(
					'id'		=> $_POST['optin_fire_id'],
					'status'	=> 'warning',
					'message'	=> $msg,
				));

				die();
			}
		}

		// Add email to Campaign Monitor

		else if ($_POST['type'] == 'campaignmonitor') {

			require_once(NNROBOTS_OPTIN_FIRE_PLUGIN_DIR . '/core/includes/campaignmonitor/csrest_subscribers.php');

			$nnrobots_optin_fire_db = new NNRobots_Optin_Fire_DB();
			$optin_fire = $nnrobots_optin_fire_db->get_data_from_id($_POST['optin_fire_id']);

			$wrap = new CS_REST_Subscribers($optin_fire['news_letter']['campaignmonitor']['list'], $optin_fire['news_letter']['campaignmonitor']['api_key']);

			// Check if subscriber is already subscribed

			$result = $wrap->get($_POST['email']);

			if ($result->was_successful()) {
				echo json_encode(array(
					'id'		=> $_POST['optin_fire_id'],
					'status'	=> 'warning',
					'message'	=> 'You are already subscribed to this list.',
				));

				die();
			}

			$result = $wrap->add(array(
				'EmailAddress' 	=> $_POST['email'],
				'Name' 			=> $_POST['first_name'] . ' ' . $_POST['last_name'],
				'Resubscribe' 	=> true
			));

			if ($result->was_successful()) {

			    // Add Cookie to let us know not to show this optin fire ever again.

				self::setEmailCookie($optin_fire['args'], NNRobots_Optin_Fire::$prefix . 'email_success_' . $_POST['optin_fire_id']);

				echo json_encode(array(
					'id'				=> $_POST['optin_fire_id'],
					'status'			=> 'check',
					'success_action'	=> $success_action,
					'url'				=> $success_url,
					'message'			=> $success_mesage,
					'conversion'		=> self::recordConversion($_POST['optin_fire_id']),
				));

				die();

			} else {

				echo json_encode(array(
					'id'		=> $_POST['optin_fire_id'],
					'status'	=> 'warning',
					'message'	=> $result->response->Message,
				));

				die();
			}
		}

		// Add email to Mad Mimi

		else if ($_POST['type'] == 'madmimi') {

			require_once(NNROBOTS_OPTIN_FIRE_PLUGIN_DIR . '/core/includes/madmimi/MadMimi.class.php');

			$nnrobots_optin_fire_db = new NNRobots_Optin_Fire_DB();
			$optin_fire = $nnrobots_optin_fire_db->get_data_from_id($_POST['optin_fire_id']);

			$mailer = new MadMimi($optin_fire['news_letter']['madmimi']['username'], $optin_fire['news_letter']['madmimi']['api_key']);

			try {

				// Check if user is already in list

				$result = $mailer->Memberships($_POST['email']);
				$lists  = new SimpleXMLElement($result);

				if ($lists->list) {
					foreach ($lists->list as $l) {
						if ($l->attributes()->{'name'}->{0} == $optin_fire['news_letter']['madmimi']['list']) {

							echo json_encode(array(
								'id'		=> $_POST['optin_fire_id'],
								'status'		=> 'check',
								'message'		=> 'You are already subscribed to this list.',
							));

							die();
						}
					}
			    }

				$result = $mailer->AddMembership($optin_fire['news_letter']['madmimi']['list'], $_POST['email'], array(
					'first_name'	=> $_POST['first_name'],
					'last_name'		=> $_POST['last_name'],
				));

				// Add Cookie to let us know not to show this optin fire ever again.

				self::setEmailCookie($optin_fire['args'], NNRobots_Optin_Fire::$prefix . 'email_success_' . $_POST['optin_fire_id']);

				echo json_encode(array(
					'id'				=> $_POST['optin_fire_id'],
					'status'			=> 'check',
					'success_action'	=> $success_action,
					'url'				=> $success_url,
					'message'			=> $success_mesage,
					'conversion'		=> self::recordConversion($_POST['optin_fire_id']),
				));

				die();

			} catch (RuntimeException $exc) {

				echo json_encode(array(
					'id'		=> $_POST['optin_fire_id'],
					'status'	=> 'warning',
					'message'	=> $msg,
				));

				die();
			}
		}

		// Add email to Infusionsoft

		else if ($_POST['type'] == 'infusionsoft') {

			require_once( NNROBOTS_OPTIN_FIRE_PLUGIN_DIR . '/core/includes/infusionsoft/isdk.php' );

			$nnrobots_optin_fire_db = new NNRobots_Optin_Fire_DB();
			$optin_fire = $nnrobots_optin_fire_db->get_data_from_id($_POST['optin_fire_id']);

			try {
				$infusion_app = new iSDK();
				$infusion_app->cfgCon( $optin_fire['news_letter']['infusionsoft']['app_id'], $optin_fire['news_letter']['infusionsoft']['api_key'], 'throw' );
			} catch( iSDKException $e ){

				echo json_encode(array(
					'id'		=> $_POST['optin_fire_id'],
					'status'	=> 'warning',
					'message'	=> $e->getMessage(),
				));

				die();
			}

			if ( empty( $error_message ) ) {

				$contact_data = $infusion_app->dsQuery( 'Contact', 1, 0, array( 'Email' => $_POST['email'] ), array( 'Id', 'Groups' ) );

				// Check if contact already exists

				if ( 0 < count( $contact_data ) ) {

					if ( false === strpos( $contact_data[0]['Groups'], $optin_fire['news_letter']['infusionsoft']['list'] ) ) {

						$infusion_app->grpAssign( $contact_data[0]['Id'], $optin_fire['news_letter']['infusionsoft']['list'] );

						// Add Cookie to let us know not to show this optin fire ever again.

						self::setEmailCookie($optin_fire['args'], NNRobots_Optin_Fire::$prefix . 'email_success_' . $_POST['optin_fire_id']);

						echo json_encode(array(
							'id'				=> $_POST['optin_fire_id'],
							'status'			=> 'check',
							'success_action'	=> $success_action,
							'url'				=> $success_url,
							'message'			=> $success_mesage,
							'conversion'		=> self::recordConversion($_POST['optin_fire_id']),
						));

						die();

					} else {

						echo json_encode(array(
							'id'			=> $_POST['optin_fire_id'],
							'status'		=> 'check',
							'message'		=> 'You are already subscribed to this list.',
						));

						die();
					}

				} else {

					$new_contact_id = $infusion_app->dsAdd( 'Contact', array(
						'FirstName' => $_POST['first_name'],
						'LastName'  => $_POST['last_name'],
						'Email'     => $_POST['email'],
					) );

					$infusion_app->grpAssign( $new_contact_id, $optin_fire['news_letter']['infusionsoft']['list'] );

					// Add Cookie to let us know not to show this optin fire ever again.

					self::setEmailCookie($optin_fire['args'], NNRobots_Optin_Fire::$prefix . 'email_success_' . $_POST['optin_fire_id']);

					echo json_encode(array(
						'id'				=> $_POST['optin_fire_id'],
						'status'			=> 'check',
						'success_action'	=> $success_action,
						'url'				=> $success_url,
						'message'			=> $success_mesage,
						'conversion'		=> self::recordConversion($_POST['optin_fire_id']),
					));

					die();
				}
			}

		}

		// Add email to MyMail

		else if ($_POST['type'] == 'mymail') {

			// Check if plugin is activated

			if ( !function_exists('mymail') ) {

				echo json_encode(array(
					'id'			=> $_POST['optin_fire_id'],
					'status'		=> 'warning',
					'message'		=> 'MyMail is not activated.',
				));

				die();

			}

			$nnrobots_optin_fire_db = new NNRobots_Optin_Fire_DB();
			$optin_fire = $nnrobots_optin_fire_db->get_data_from_id($_POST['optin_fire_id']);

			// Add subscriber

			$subscriber_id = mymail('subscribers')->add(array(
				'email' 	=> $_POST['email'],
				'firstname'	=> $_POST['first_name'],
				'lastname'	=> $_POST['last_name'],
				'referer'	=> 'Optin Fire',
			), false);

			// Add to List

	        if ( !is_wp_error($subscriber_id) ) {

	            mymail('subscribers')->assign_lists($subscriber_id, array($optin_fire['news_letter']['mymail']['list']));

	            // Add Cookie to let us know not to show this optin fire ever again.

				self::setEmailCookie($optin_fire['args'], NNRobots_Optin_Fire::$prefix . 'email_success_' . $_POST['optin_fire_id']);

				echo json_encode(array(
					'id'				=> $_POST['optin_fire_id'],
					'status'			=> 'check',
					'success_action'	=> $success_action,
					'url'				=> $success_url,
					'message'			=> $success_mesage,
					'conversion'		=> self::recordConversion($_POST['optin_fire_id']),
				));

				die();

	        } else {

	        	echo json_encode(array(
					'id'			=> $_POST['optin_fire_id'],
					'status'		=> 'check',
					'message'		=> 'You are already subscribed to this list.',
				));

				die();

	        }

		}

		// Add email to Active Campaign

		else if ($_POST['type'] == 'activecampaign') {

			require_once( NNROBOTS_OPTIN_FIRE_PLUGIN_DIR . '/core/includes/activecampaign/ActiveCampaign.class.php' );

			$nnrobots_optin_fire_db = new NNRobots_Optin_Fire_DB();
			$optin_fire = $nnrobots_optin_fire_db->get_data_from_id($_POST['optin_fire_id']);

			$ac = new ActiveCampaign($optin_fire['news_letter']['activecampaign']['app_url'], $optin_fire['news_letter']['activecampaign']['api_key']);

			if ( !(int)$ac->credentials_test() ) {

				echo json_encode(array(
					'id'		=> $_POST['optin_fire_id'],
					'status'	=> 'warning',
					'message'	=> __('Unable to to connect to Active Campaign.', NNRobots_Optin_Fire::$text_domain)
				));

				die();
			}

			// Add subscriber

			$contact_sync = $ac->api("contact/add", array(
				"email"             													=> $_POST['email'],
				"first_name"         													=> $_POST['first_name'],
				"last_name"          													=> $_POST['last_name'],
				"p[" . $optin_fire['news_letter']['activecampaign']['list'] . "]"      	=> $optin_fire['news_letter']['activecampaign']['list'],
				"status[" . $optin_fire['news_letter']['activecampaign']['list'] . "]" 	=> 1, // "Active" status
			));

			if ( (int) $contact_sync->success ) {

				// Add Cookie to let us know not to show this optin fire ever again.

				self::setEmailCookie($optin_fire['args'], NNRobots_Optin_Fire::$prefix . 'email_success_' . $_POST['optin_fire_id']);

				echo json_encode(array(
					'id'				=> $_POST['optin_fire_id'],
					'status'			=> 'check',
					'success_action'	=> $success_action,
					'url'				=> $success_url,
					'message'			=> $success_mesage,
					'conversion'		=> self::recordConversion($_POST['optin_fire_id']),
				));

				die();

			} else {

				echo json_encode(array(
					'id'		=> $_POST['optin_fire_id'],
					'status'	=> 'warning',
					'message'	=> $contact_sync->error,
				));

				die();
			}
		}

		echo json_encode(array(
			'id'		=> $_POST['optin_fire_id'],
			'status'	=> 'warning',
			'message'	=> __('Unable to subscribe user. Newsletter not setup properly.', NNRobots_Optin_Fire::$text_domain)
		));

		die(); // this is required to terminate immediately and return a proper response
	}

	/**
	 * Deletes the stats for a optin fire when the optin fire is deleted
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function deleteEmail($email) {

		// Return false if optin_fire_id is not set

		if (!isset($email)) {
			return false;
		}

		global $wpdb;

		// Check if entry already exsits

		$query = $wpdb->prepare('DELETE FROM ' . self::getTableName() . ' WHERE `email` = %d', $email);
		$result = $wpdb->query($query);

		return $result;

	}

	/**
	 * Record a converison made by a newsletter signup
	 *
	 * @access public
	 * @static
	 * @param mixed $optin_fire_id
	 * @return void
	 */
	static function recordConversion($optin_fire_id) {

		include_once(NNROBOTS_OPTIN_FIRE_PLUGIN_DIR . '/core/model/stats.php');

		// Return if no data is given

		if (!isset($optin_fire_id)) {
			return 'Optin conversion was NOT able to be recored because the Optin ID was not retrieved.';
		}

		// Return if user is admin or higher

		if (current_user_can('edit_published_posts')) {
			return 'Optin conversion was NOT able to be recored because current user is not authorized to log conversions.';
		}

		// Return false if crawler

		if (NNRobots_Stats_OF::isBot()) {
			return 'Optin conversion was NOT able to be recored because there is no user.  A crawler was detected.';
		}

		global $wpdb;
		$today = date(NNROBOTS_OPTIN_FIRE_DATE_FORMAT);

		// Check if entry already exsits

		$query = $wpdb->prepare('SELECT * FROM ' . NNRobots_Stats_OF::getTableName() . ' WHERE `date` = %s AND `optin_fire_id` = %d', $today, $optin_fire_id);
		$optin_fire_conversions = $wpdb->query($query);

		// Entry aleady exists, just add 1

		if (isset($optin_fire_conversions) && $optin_fire_conversions != 0) {

			$query = $wpdb->prepare('UPDATE ' . NNRobots_Stats_OF::getTableName() . ' SET
				`conversions` = `conversions` + 1
				WHERE `date` = %s AND `optin_fire_id` = %d', $today, $optin_fire_id);

			$result = $wpdb->query($query);

		}

		// Entry does not exist, create a new one and set impressions to 1

		else {
			$query = $wpdb->prepare('INSERT INTO ' . NNRobots_Stats_OF::getTableName() . ' (
				`date`,
				`optin_fire_id`,
				`impressions`,
				`conversions`
				) VALUES (%s, %d, 0, 1)',
				$today,
				$optin_fire_id
			);

			$result = $wpdb->query($query);
		}

		if ($result) {
			return 'Optin conversion was able to be recored.';
		} else {
			return 'Optin conversion was NOT able to be recored.';
		}

		return '';
	}

	/**
	 * Set the cookie for your browser to not show an optin fire
	 *
	 * @access public
	 * @static
	 * @param mixed $name
	 * @return void
	 */
	static function setEmailCookie($args, $name){

		if ($args['show_after_success'] == 'no') {
			setcookie($name, true, 0, '/');
		}

	}

	/**
	 * Returns the proper table name based off of site ID
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function getTableName(){

		global $wpdb;

		return '`' . $wpdb->prefix . self::$table_name . '`';
	}

}

//endif;