<?php

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) exit;

// Check if class already exists

//if (!class_exists("NNRobots_Stats_OF")):

// AJAX functions for impression and click tracking

add_action( 'wp_ajax_nnr_of_record_impression', 		array('NNRobots_Stats_OF', 'recordImpresssion'));
add_action( 'wp_ajax_nopriv_nnr_of_record_impression', 	array('NNRobots_Stats_OF', 'recordImpresssion'));
add_action( 'wp_ajax_nnr_of_record_click', 				array('NNRobots_Stats_OF', 'recordConversion'));
add_action( 'wp_ajax_nopriv_nnr_of_record_click', 		array('NNRobots_Stats_OF', 'recordConversion'));

/**
 * NNRobots_Stats_OF class.
 *
 * This class is used to record all stats for optin fires.
 */
class NNRobots_Stats_OF {

	/**
	 * table_name
	 *
	 * (default value: 'nnrobots_optin_fire_stats')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	public static $table_name = 'nnrobots_optin_fire_stats';

	/**
	 * Create Table when this class is loaded
	 *
	 * @access public
	 * @return void
	 */
	function __construct() {
		self::createTable();
	}

	/**
	 * Creates the Stats table if it does not exist
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function createTable(){

		global $wpdb;

		$wpdb->query("
			CREATE TABLE IF NOT EXISTS `" . self::getTableName() . "` (
				`id` int(11) NOT NULL AUTO_INCREMENT,
				`date` date NOT NULL,
				`optin_fire_id` int(11) NOT NULL DEFAULT 0,
				`impressions` int(8) NOT NULL DEFAULT 0,
				`conversions` int(8) NOT NULL DEFAULT 0,
				PRIMARY KEY (`id`)
			) ENGINE=InnoDB DEFAULT CHARSET=latin1 AUTO_INCREMENT=1;
		");

	}

	/**
	 * Get the stats for all optin fires in a timeframe or a single firbar in a timeframe
	 *
	 * @access public
	 * @static
	 * @param mixed $start
	 * @param mixed $end
	 * @param mixed $id (default: null)
	 * @return void
	 */
	static function getStats($start, $end, $id = null, $select = '*') {

		$query = null;

		global $wpdb;

		// All Optins, All Time

		if ($start == null && $end == null && $id == null) {
			$query = 'SELECT ' . $select . ' FROM ' . self::getTableName();
		}

		// All Optins, After Date

		else if ($start != null && $end == null && $id == null) {
			$query = $wpdb->prepare('SELECT ' . $select . ' FROM ' . self::getTableName() . ' WHERE `date` = %s', $start);
		}

		// All Optins, Date Range

		else if ($start != null && $end != null && $id == null) {
			$query = $wpdb->prepare('SELECT ' . $select . ' FROM ' . self::getTableName() . ' WHERE `date` >= %s AND `date` <= %s', $start, $end);
		}

		// Single Optin, All Time

		else if ($start == null && $end == null && $id != null) {
			$query = $wpdb->prepare('SELECT ' . $select . ' FROM ' . self::getTableName() . ' WHERE `optin_fire_id` = %d', $id);
		}

		// Single Optin, Date Range

		else if ($start != null && $end != null && $id != null) {
			$query = $wpdb->prepare('SELECT ' . $select . ' FROM ' . self::getTableName() . ' WHERE `date` >= %s AND `date` <= %s AND `optin_fire_id` = %d', $start, $end, $id);
		}

		// Single Optin, After Date

		else if ($start != null && $end == null && $id != null) {
			$query = $wpdb->prepare('SELECT ' . $select . ' FROM ' . self::getTableName() . ' WHERE `date` >= %s AND `optin_fire_id` = %d', $start, $id);
		}

		// No query was created

		if (!isset($query)) {
			return false;
		}

		$result = $wpdb->get_results($query, 'ARRAY_A');

		return $result;
	}

	/**
	 * Runs everyday and will add a record to table if none exists
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function initialize_optin_fire_stats() {

		global $wpdb;
		$today = date(NNROBOTS_OPTIN_FIRE_DATE_FORMAT);

		include_once(NNROBOTS_OPTIN_FIRE_PLUGIN_DIR . "/admin/model/db.php");

		$optin_fires = $wpdb->get_results("SELECT * FROM `" . NNRobots_Optin_Fire_DB::getTableName() . "`");

		$result = false;

		foreach ($optin_fires as $optin_fire) {

			// Check if entry already exsits

			$query = $wpdb->prepare('SELECT * FROM ' . self::getTableName() . ' WHERE `date` = %s AND `optin_fire_id` = %d', $today, $optin_fire->id);
			$optin_fire_impressions = $wpdb->query($query);

			// Entry does not exist, create a new one and set impressions to 0 and conversions to 0

			if (!isset($optin_fire_impressions) || $optin_fire_impressions == 0) {

				$query = $wpdb->prepare('INSERT INTO ' . self::getTableName() . ' (
					`date`,
					`optin_fire_id`,
					`impressions`,
					`conversions`
					) VALUES (%s, %d, 0, 0)',
					$today,
					$optin_fire->id
				);

				$result = $wpdb->query($query);
			}
		}

		return $result;

	}

	/**
	 * Record an impression made by a optin_fire
	 * This function is called via PHP.
	 *
	 * @access public
	 * @static
	 * @param mixed $data (default: null)
	 * @return void
	 */
	static function recordImpresssion($data = null) {

		$data['optin_fire_id'] = $_POST['optin_fire_id'];

		// Return false if no data is given

		if (!isset($data) || !isset($data['optin_fire_id'])) {
			echo 'Optin impression was NOT able to be recored because no optin_fire id given.';
			die();
		}

		// Return false if crawler

		if (self::isBot()) {
			echo 'Optin impression was NOT able to be recored because a crawler was detected.';
			die();
		}

		// Return if user is admin or higher

		if (current_user_can('edit_published_posts')) {
			echo 'Optin impression was NOT able to be recored because current user is an admin.';
			die();
		}

		global $wpdb;
		$today = date(NNROBOTS_OPTIN_FIRE_DATE_FORMAT);

		// Check if entry already exsits

		$query = $wpdb->prepare('SELECT * FROM ' . self::getTableName() . ' WHERE `date` = %s AND `optin_fire_id` = %d', $today, $data['optin_fire_id']);
		$optin_fire_impressions = $wpdb->query($query);

		// Entry aleady exists, just add 1

		if (isset($optin_fire_impressions) && $optin_fire_impressions != 0) {

			$query = $wpdb->prepare('UPDATE ' . self::getTableName() . ' SET
				`impressions` = `impressions` + 1
				WHERE `date` = %s AND `optin_fire_id` = %d', $today, $data['optin_fire_id']);

			$result = $wpdb->query($query);

		}

		// Entry does not exist, create a new one and set impressions to 1

		else {
			$query = $wpdb->prepare('INSERT INTO ' . self::getTableName() . ' (
				`date`,
				`optin_fire_id`,
				`impressions`,
				`conversions`
				) VALUES (%s, %d, 1, 0)',
				$today,
				$data['optin_fire_id']
			);

			$result = $wpdb->query($query);
		}

		if ($result) {
			echo 'Optin impression was able to be recored.';
		} else {
			echo 'Optin impression was NOT able to be recored.';
		}

		die(); // necessary for WordPress AJAX calls

	}

	/**
	 * Record a click made by a optin fire
	 * This function is called via AJAX
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function recordConversion() {

		$optin_fire_id = $_POST['optin_fire_id'];

		// Return if no data is given

		if (!isset($optin_fire_id)) {
			echo 'Optin click was NOT able to be recored because the Optin ID was not retrieved.';
			die();
		}

		// Return if user is admin or higher

		if (current_user_can('edit_published_posts')) {
			echo 'Optin click was NOT able to be recored because current user is not authorized to log conversions.';
			die();
		}

		// Return false if crawler

		if (self::isBot()) {
			echo 'Optin click was NOT able to be recored because there is no user.  A crawler was detected';
			die();
		}

		global $wpdb;
		$today = date(NNROBOTS_OPTIN_FIRE_DATE_FORMAT);

		// Check if entry already exsits

		$query = $wpdb->prepare('SELECT * FROM ' . self::getTableName() . ' WHERE `date` = %s AND `optin_fire_id` = %d', $today, $optin_fire_id);
		$optin_fire_conversions = $wpdb->query($query);

		// Entry aleady exists, just add 1

		if (isset($optin_fire_conversions) && $optin_fire_conversions != 0) {

			$query = $wpdb->prepare('UPDATE ' . self::getTableName() . ' SET
				`conversions` = `conversions` + 1
				WHERE `date` = %s AND `optin_fire_id` = %d', $today, $optin_fire_id);

			$result = $wpdb->query($query);

		}

		// Entry does not exist, create a new one and set impressions to 1

		else {
			$query = $wpdb->prepare('INSERT INTO ' . self::getTableName() . ' (
				`date`,
				`optin_fire_id`,
				`impressions`,
				`conversions`
				) VALUES (%s, %d, 0, 1)',
				$today,
				$optin_fire_id
			);

			$result = $wpdb->query($query);
		}

		if ($result) {
			echo 'Optin click was able to be recored.';
		} else {
			echo 'Optin click was NOT able to be recored.';
		}

		die(); // necessary for WordPress AJAX calls
	}

	/**
	 * Deletes the stats for a optin fire when the optin fire is deleted
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function deleteStats($optin_fire_id) {

		// Return false if optin_fire_id is not set

		if (!isset($optin_fire_id)) {
			return false;
		}

		global $wpdb;

		// Check if entry already exsits

		$query = $wpdb->prepare('DELETE FROM ' . self::getTableName() . ' WHERE `optin_fire_id` = %d', $optin_fire_id);
		$result = $wpdb->query($query);

		return $result;

	}

	/**
	 * Check for BOTs visiting site
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function isBot() {

		if (isset($_SERVER['HTTP_USER_AGENT']) && preg_match('/bot|spider|crawler|curl|^$/i', $_SERVER['HTTP_USER_AGENT'])) {
			return true;
		} else {
			return false;
		}

	}

	/**
	 * Returns the proper table name based off of site ID
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function getTableName(){

		global $wpdb;

		return $wpdb->prefix . self::$table_name;
	}

}

//endif;