<?php

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) exit;

add_action( 'widgets_init', function(){
     register_widget( 'NNRobots_Optin_Fire_Widget' );
});

//if ( !class_exists('NNRobots_Optin_Fire_Widget') ) :

/**
 * Adds NNRobots_Popular_Posts_Widget widget.
 */
class NNRobots_Optin_Fire_Widget extends WP_Widget {

	/**
	 * Register widget with WordPress.
	 */
	function __construct() {
		parent::__construct(
			'nnr-optin-fire', // Base ID
			__( '99 Robots Optin Fire', NNRobots_Optin_Fire::$text_domain ), // Name
			array( 'description' => __( 'Displays an Optin Fire', NNRobots_Optin_Fire::$text_domain ), ) // Args
		);
	}

	/**
	 * Front-end display of widget.
	 *
	 * @see WP_Widget::widget()
	 *
	 * @param array $args     Widget arguments.
	 * @param array $instance Saved values from database.
	 */
	public function widget( $args, $instance ) {

		if (isset($instance['optin_fire_id']) && $instance['optin_fire_id'] != '') {

			include_once(NNROBOTS_OPTIN_FIRE_PLUGIN_DIR . "/admin/model/db.php");
			include_once(NNROBOTS_OPTIN_FIRE_PLUGIN_DIR . "/core/functions.php");

			global $wpdb;

			// Get all Optins

			$nnrobots_optin_fire_db = new NNRobots_Optin_Fire_DB();
			$optin_fire = $nnrobots_optin_fire_db->get_data_from_id($instance['optin_fire_id']);

			// Check if we got an optin back

			if ( isset($optin_fire) && $optin_fire['active'] == 'Active' ) {

				// Check that conditions are true

				if (NNRobots_Optin_Fire_Functions::check_conditions($optin_fire, $optin_fire['template'], $optin_fire['args'], $optin_fire['trigger'])) {

					echo $args['before_widget'];
					if ( ! empty( $instance['title'] ) ) {
						echo $args['before_title'] . apply_filters( 'widget_title', $instance['title'] ). $args['after_title'];
					}
		    		echo NNRobots_Optin_Fire_Functions::generate_code($optin_fire, $optin_fire['news_letter'], true);
		    		echo $args['after_widget'];
				}
			}
		}
	}

	/**
	 * Back-end widget form.
	 *
	 * @see WP_Widget::form()
	 *
	 * @param array $instance Previously saved values from database.
	 */
	public function form( $instance ) {

		include_once(NNROBOTS_OPTIN_FIRE_PLUGIN_DIR . "/admin/model/db.php");

		global $wpdb;

		// Get all Optins

		$nnrobots_optin_fire_db = new NNRobots_Optin_Fire_DB();
		$optin_fires =  $nnrobots_optin_fire_db->get_by_type(array(4,5));

		?>
		<p>
			<label for="<?php echo $this->get_field_id( 'title' ); ?>"><?php _e( 'Title:' ); ?></label>
			<input class="widefat" id="<?php echo $this->get_field_id( 'title' ); ?>" name="<?php echo $this->get_field_name( 'title' ); ?>" type="text" value="<?php echo (isset($instance['title']) ? $instance['title'] : ''); ?>"/>
		</p>

		<p>
			<label for="<?php echo $this->get_field_id( 'optin-fire-id' ); ?>"><?php _e( 'Optin:' ); ?></label>
			<select class="widefat" id="<?php echo $this->get_field_id( 'optin-fire-id' ); ?>" name="<?php echo $this->get_field_name( 'optin-fire-id' ); ?>">

				<?php foreach ($optin_fires as $optin_fire) {

					// Check if this is a variant

					if ( isset($optin_fire['args']['ab_testing']['active']) && $optin_fire['args']['ab_testing']['active'] &&
						isset($optin_fire['args']['ab_testing']['is_variant']) && $optin_fire['args']['ab_testing']['is_variant'] ) {
						continue;
					} ?>
					<option value="<?php echo $optin_fire['id']; ?>" <?php selected($optin_fire['id'], $instance['optin_fire_id'], true); ?>><?php echo $optin_fire['name']; ?></option>
				<?php } ?>

			</select>
		</p>
		<?php
	}

	/**
	 * Sanitize widget form values as they are saved.
	 *
	 * @see WP_Widget::update()
	 *
	 * @param array $new_instance Values just sent to be saved.
	 * @param array $old_instance Previously saved values from database.
	 *
	 * @return array Updated safe values to be saved.
	 */
	public function update( $new_instance, $old_instance ) {
		$instance = array();
		$instance['title'] = ( ! empty( $new_instance['title'] ) ) ? strip_tags( $new_instance['title'] ) : '';
		$instance['optin_fire_id'] = ( ! empty( $new_instance['optin-fire-id'] ) ) ? strip_tags( $new_instance['optin-fire-id'] ) : '';

		return $instance;
	}

}

//endif;